/* ------------------------------------------------------------------------------------------------ */
/* FILE :		Demo_PANDA_M3_A9_Main_CCS.c															*/
/*																									*/
/* CONTENTS :																						*/
/*				program to run on the CortexA9_0 for the Demo_3_SMP_PANDA_M3_CSS					*/
/*																									*/
/* Most of the code originates from the supplied OMAP4460 gel files									*/
/* Except, instead of configuring the Cortex M3 AMMU for 16K, it is configured for 64K				*/
/*																									*/
/*																									*/
/* Copyright (c) 2012-2013, Code-Time Technologies Inc. All rights reserved.						*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.5 $																				*/
/*	$Date: 2013/06/08 22:17:37 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#include "stdio.h"

#define CORTEXM3_RTOS_IN_RESET   1
#define CORTEXM3_SIMCOP_IN_RESET 2
#define CORTEXM3_0_RESET_MASK    3
#define CORTEXM3_1_RESET_MASK    3
#define CORTEXM3_BOTH_RESET_MASK 3

void reset_ducati_cortexM3_SIMCOP_board(void);
void reset_ducati_cortexM3_RTOS_board(void);
void reset_tesla_C64_board(void);

volatile int Dummy;

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

main()
{
volatile int *ClkReg;								/* Pointer to re-program the MPU & PER clocks	*/

	reset_ducati_cortexM3_RTOS_board();
	reset_ducati_cortexM3_SIMCOP_board();
	reset_tesla_C64_board();

	ClkReg = (volatile int *)0x4A004100;			/* Base address of the MPU clock resgisters		*/

	ClkReg[0x60/4] = (0xFFFFFFF8 & ClkReg[0x60/4])	/* CM_CLKMODE_DPLL_MPU: put DPLL in bypass		*/
	               |  0x00000004;
	while(0x1 & ClkReg[0x64/4]);					/* CM_IDLE_TEST_DPLL_MPU: wait for DPLL bypass	*/
	ClkReg[0x00/4] &= 0xFFFFFFFE;					/* CORE_X2_CLK / 1								*/
	ClkReg[0x68/4] &= 0xFFFFFFF8;					/* CM_AUTO_IDLE_DPLL_MPU						*/
	ClkReg[0x68/4] |= 0x00000004;
	ClkReg[0x6C/4] = (73<<8)						/* CM_CLKSEL_DPLL_MPU: Set div & mult			*/
	               | ( 3<<0);						/* 700MHz = 38.4MHz * 73 / (3+1)				*/
	ClkReg[0x70/4] = (1<<8)							/* CM_DIV_M2_DPLL_MPU: divide by 1				*/
	               | (1<<0);
	ClkReg[0x60/4] = (0xFFFFFFF8 & ClkReg[0x60/4])	/* CM_CLKMODE_DPLL_MPU: lock the DPLL			*/
	               |  0x00000007;
	while(!(0x1 & ClkReg[0x64/4]));					/* CM_IDLE_TEST_DPLL_MPU: wait for lock			*/
													/* -------------------------------------------- */

	ClkReg = (volatile int *)0x4A008100;			/* Base address of the PER clock resgisters		*/
	ClkReg[0x40/4] = (0xFFFFFFF8 & ClkReg[0x40/4])	/* CM_IDLEST_DPLL_PER: put DPLL in bypas		*/
	               |  0x00000004;
	while(0x1 & ClkReg[0x44/4]);					/* CM_IDLE_TEST_DPLL_PER: wait for DPLL bypass	*/
	ClkReg[0x00/4] |= 0x00000001;					/* M3 clock is from DPLL_PER					*/ 
	ClkReg[0x4C/4] = (195<<8)						/* CM_CLKSEL_DPLL_PER: Set div & mult			*/
                   | ( 38<<0);						/* 192MHz = 38.4MHz * 195 / (38+1)				*/
	ClkReg[0x50/4] = (1<<8)							/* CM_DIV_M2_DPLL_MPU: divide by 2				*/
	               | (2<<0);						/* /2 because M3 is tied to X2 clock			*/
	ClkReg[0x60/4] = (0xFFFFFFE0 & ClkReg[0x60/4])	/* M3s clock /1									*/
	               |  0x00000001;
	ClkReg[0x40/4] = (0xFFFFFFF8 & ClkReg[0x40/4])	/* CM_IDLEST_DPLL_PER: lock the DPLL			*/
	               |  0x00000007;
	while(!(0x1 & ClkReg[0x44/4]));					/* CM_IDLE_TEST_DPLL_PER: wait for lock			*/


	puts("\n********************************************");
	puts("The Cortex M3s are ready for download");
	puts("Connect to both Cortex M3s:");
	puts("    Select Cortex_M3_0");
	puts("    Pull down menu: Run / Connect Target");
	puts("    Select Cortex_M3_1");
	puts("    Pull down menu: Run / Connect Target");
	puts("Download the application on Cortex_M3_0:");
	puts("    Select Cortex_M3_0");
	puts("    Pull down menu Run / Load / Load Program");
	puts("    Program: ../../Demo_3_SMP_PANDA_M3_CCS/Debug/Demo_3_SMP_PANDA_M3_CCS.out");
	puts("Resume Cortex_M3_0");
	puts("For SMP, after a few seconds, the selected CPU switches to Cortex_M3_1:");
	puts("        Resume Cortex_M3_1");

	for(;;);
}

 
/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

void reset_ducati_cortexM3_RTOS_board(void)
{
int cortexResetRegister;
int cortexResetStatus;

	cortexResetRegister = *(int*)0x4A306910;
	cortexResetStatus   = cortexResetRegister
	                    & CORTEXM3_0_RESET_MASK;
													/* Check if both devices are in reset			*/
	if (cortexResetStatus == (CORTEXM3_RTOS_IN_RESET | CORTEXM3_SIMCOP_IN_RESET) ) {
		*(volatile int*)0x4A008920 = 0x00000001;	/* Configure hte Ducati in Auto mode			*/
													/* Reg CORE_CM2:CM_DUCATI_DUCATI_CLKCTRL		*/
		*(volatile int*)0x4A008900 = 0x00000002;	/* Set force-wakeup domain transition			*/
													/* Reg CORE_CM2:CM_DUCATI_CLKSTCTRL = SW_WKUP	*/
		while (((*(volatile int*)0x4A008900) & 0x100)& ~0x100);	/* wait until the clock is running	*/
	}
													/* CORTEXM3_0 is in reset						*/
	if ( (cortexResetStatus & CORTEXM3_RTOS_IN_RESET) != 0 ) {
													/* CORTEXM3_1 is in reset						*/
		if ((cortexResetStatus & CORTEXM3_SIMCOP_IN_RESET)!= 0) {
			cortexResetRegister = cortexResetRegister
			                    & 0xFFFFFFFB;
   			*(volatile int*)0x4A306910   = cortexResetRegister;	/* Release Ducati MMU/Cache IF reset*/
   			while (((*(volatile int*)0x4A306914) & 0x4)& ~0x4);	/* Wait for reset applied	*/ 
   			*(volatile int*)0x4A306914 = 0x00000004;/* Reset status cleared							*/
    	}
													/* Boot from Ducati L2RAM - self loop			*/
													/* Overwrite reset values for AMMU smallPage	*/
													/* policy registers. Setting for 64 K			*/
		*(volatile unsigned int*)0x55080920=0x00000000;
		*(volatile unsigned int*)0x55080924=0x40000000;
		*(volatile unsigned int*)0x55080928=0x00004000;
		*(volatile unsigned int*)0x5508092C=0x00008000;
		*(volatile unsigned int*)0x55080930=0x0000C000;

		*(volatile unsigned int*)0x550809A0=0x55020000;
		*(volatile unsigned int*)0x550809A4=0x55080000;
		*(volatile unsigned int*)0x550809A8=0x55024000;
		*(volatile unsigned int*)0x550809AC=0x55028000;
		*(volatile unsigned int*)0x550809B0=0x5502C000;

		*(volatile unsigned int*)0x55080A20=0x0001000B;
		*(volatile unsigned int*)0x55080A24=0x0000000B;
		*(volatile unsigned int*)0x55080A28=0x00010007;
		*(volatile unsigned int*)0x55080A2C=0x00000007;
		*(volatile unsigned int*)0x55080A30=0x00000007;

		*(volatile int*)0x550809A0 = 0x55020000;	/* Configure Ducati to boot from Ducati L2RAM	*/
													/* Reg AMMU Small Page0 XLTE Register			*/
		*(volatile int*)0x55020004 = 0x0000FF01;	/* Reset initial PC								*/
		*(volatile int*)0x5502FF00 = 0xE7FEE7FE;	/* Branch to itself @ 0xFF00					*/

		cortexResetRegister = cortexResetRegister
		                    & (~(CORTEXM3_0_RESET_MASK));
   		*(volatile int*)0x4A306910   = cortexResetRegister;	/* Release Ducati CortexM3 RTOS			*/

   		while (((*(volatile int*)0x4A306914) & 0x1)& ~0x1);	/* Wait for Ducati RTOS reset applied	*/

   		*(volatile int*)0x4A306914 = 0x00000001;	/* Clear reset status							*/
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

void reset_ducati_cortexM3_SIMCOP_board(void)
{
int cortexResetRegister;
int cortexResetStatus;

	cortexResetRegister = *(int*)0x4A306910;
	cortexResetStatus   = cortexResetRegister
	                    & CORTEXM3_1_RESET_MASK;

	if (cortexResetStatus == (CORTEXM3_RTOS_IN_RESET | CORTEXM3_SIMCOP_IN_RESET) ) {
		*(volatile int*)0x4A008920 = 0x00000001;	/* Configure hte Ducati in Auto mode			*/
													/* Reg CORE_CM2:CM_DUCATI_DUCATI_CLKCTRL		*/
		*(volatile int*)0x4A008900 = 0x00000002;	/* Set force-wakeup domain transition			*/
													/* Reg CORE_CM2:CM_DUCATI_CLKSTCTRL = SW_WKUP	*/
		while (((*(volatile int*)0x4A008900) & 0x100)& ~0x100);	/* Wait until the clock is running	*/
	}

	if ((cortexResetStatus & CORTEXM3_SIMCOP_IN_RESET)!= 0) {
		if ( (cortexResetStatus & CORTEXM3_RTOS_IN_RESET) != 0 ) {
			cortexResetRegister = cortexResetRegister
			                    & 0xFFFFFFFB; 	
   			*(volatile int*)0x4A306910   = cortexResetRegister;	/* Release Ducati MMU/Cache IF reset*/
   			while (((*(volatile int*)0x4A306914) & 0x4)& ~0x4);	/* Wait for reset applied			*/ 
   			*(volatile int*)0x4A306914 = 0x00000004;/* Reset status cleared							*/
   		}
		cortexResetRegister = cortexResetRegister	/* Release Ducati CortexM3 SIMCOP				*/
		                    & (~(CORTEXM3_1_RESET_MASK));
   		*(volatile int*)0x4A306910   = cortexResetRegister;
   		while (((*(volatile int*)0x4A306914) & 0x2)& ~0x2);	/* Wait Ducati SIMCOP reset applied		*/
   		*(volatile int*)0x4A306914 = 0x00000002;	/* Clear reset status							*/
   		
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

void reset_tesla_C64_board(void)
{
int ii;
													/* IVAHD WAKEUP (after IVAHD dpll)				*/
	*(volatile int*)0x4A008F20 = 0x00000001;		/* Configure IVAHD/SL2 modules into "Auto" mode	*/
													/* Reg IVAHD_CM2:CM_IVAHD_IVAHD_CLKCTRL			*/
	*(volatile int*)0x4A008F28 = 0x00000001;		/* Reg IVAHD_CM2:CM_IVAHD_SL2_CLKCTRL			*/
	*(volatile int*)0x4A008F00 = 0x00000002;		/* Set Force wakeUp domain transition			*/
													/* Reg IVAHD_CM2:CM_IVAHD_CLKSTCTRL = SW_WKUP	*/
	*(volatile int*)0x4A004420 = 0x00000001;		/* Configure Tesla mode into Auto mode			*/
													/* Reg TESLA_CM1:CM_TESLA_TESLA_CLKCTRL			*/

	*(volatile int*)0x4A004400 = 0x00000002;		/* Set force-wakeup domain transition			*/

	for(ii=0 ; ii<10000 ; ii++) {Dummy++;}			/* Need some delays here and there				*/

													/* TESLA_CM1:CM_TESLA_CLKSTCTRL = SW_WKUP		*/
	while (((*(volatile int*)0x4A004400) & 0x100)& ~0x100);	/* Wait until the clock is running		*/

	*(volatile int*)0x4A306F10 &= ~(1 << 2);		/*elease slave port reset for IVA-HD allows		*/
													/* SL2 access									*/
	for(ii=0 ; ii<10000 ; ii++) {Dummy++;}			/* Need some delays here and there				*/

 	*(volatile int*)0x5B000000 = 0x00;
	*(volatile int*)0x5B000004 = 0x00;
	*(volatile int*)0x5B000008 = 0x00;
	*(volatile int*)0x5B00000C = 0x00;
	*(volatile int*)0x5B000010 = 0x00;
	*(volatile int*)0x5B000014 = 0x00;
	*(volatile int*)0x5B000018 = 0x00;
	*(volatile int*)0x5B00001c = 0x00;
	*(volatile int*)0x5B000020 = 0x12;
	*(volatile int*)0x5B000024 = 0x00;
	*(volatile int*)0x5B000028 = 0x00;
	*(volatile int*)0x5B00002C = 0x00;
	*(volatile int*)0x5B000030 = 0x00;
	*(volatile int*)0x5B000034 = 0x00;
	*(volatile int*)0x5B000038 = 0x00;

	*(volatile int*)0x4A002304 = 0x10800000;  		/* Set tesla boot address						*/
	*(volatile int*)0x4A306410 = 0x00000000;		/* DSP reset									*/

	for(ii=0 ; ii<10000 ; ii++) {Dummy++;}			/* Need some delays here and there				*/
													/* Reg RM_TESLA_RSTCTRL (MMU and DSP RESET)		*/
	while (((*(volatile int*)0x4A306414) & 0x0001)& ~0x0001);	/* wait Tesla reset applied 		*/
	while (((*(volatile int*)0x4A306F14) & 0x0004)& ~0x0004);	/* wait slave port reset 	done	*/
	*(volatile int*)0x4A306F14 = 0x4;				/* Clear reset status 							*/

	for(ii=0 ; ii<10000 ; ii++) {Dummy++;}			/* Need some delays here and there				*/

	while (((*(volatile int*)0x4A004420) & 0x30000)& ~0x30000);	/* Read 0x0 : test that module is OK*/

	return;
}

/* EOF */
